@@ -1,5 +1,5 @@
 name = "labelbox"
-__version__ = "3.26.1"
+__version__ = "3.40.1"
 
 from labelbox.client import Client
 from labelbox.schema.project import Project
@@ -20,10 +20,13 @@ from labelbox.schema.webhook import Webhook
 from labelbox.schema.ontology import Ontology, OntologyBuilder, Classification, Option, Tool, FeatureSchema
 from labelbox.schema.role import Role, ProjectRole
 from labelbox.schema.invite import Invite, InviteLimit
-from labelbox.schema.data_row_metadata import DataRowMetadataOntology
+from labelbox.schema.data_row_metadata import DataRowMetadataOntology, DataRowMetadataField, DataRowMetadata, DeleteDataRowMetadata
 from labelbox.schema.model_run import ModelRun, DataSplit
 from labelbox.schema.benchmark import Benchmark
 from labelbox.schema.iam_integration import IAMIntegration
 from labelbox.schema.resource_tag import ResourceTag
 from labelbox.schema.project_resource_tag import ProjectResourceTag
 from labelbox.schema.media_type import MediaType
+from labelbox.schema.slice import Slice, CatalogSlice, ModelSlice
+from labelbox.schema.queue_mode import QueueMode
+from labelbox.schema.task_queue import TaskQueue
