@@ -2,9 +2,10 @@ from typing import Callable, Optional, Tuple, Union, Dict, List
 
 import numpy as np
 from pydantic.class_validators import validator
-from shapely.geometry import MultiPolygon, Polygon
 import cv2
 
+from shapely.geometry import MultiPolygon, Polygon
+
 from ..data import MaskData
 from .geometry import Geometry
 
@@ -54,6 +55,13 @@ class Mask(Geometry):
         external_polygons = self._extract_polygons_from_contours(
             external_contours)
         holes = self._extract_polygons_from_contours(holes)
+
+        if not external_polygons.is_valid:
+            external_polygons = external_polygons.buffer(0)
+
+        if not holes.is_valid:
+            holes = holes.buffer(0)
+
         return external_polygons.difference(holes).__geo_interface__
 
     def draw(self,
@@ -78,7 +86,6 @@ class Mask(Geometry):
             np.ndarray representing only this object
                 as opposed to the mask that this object references which might have multiple objects determined by colors
         """
-
         mask = self.mask.value
         mask = np.alltrue(mask == self.color, axis=2).astype(np.uint8)
 
