@@ -36,8 +36,12 @@ class _TypedArray(np.ndarray, Generic[DType, DShape]):
         return val
 
 
-if version.parse(np.__version__) >= version.parse('1.22.0'):
+if version.parse(np.__version__) >= version.parse('1.23.0'):
+    from numpy._typing import _GenericAlias
+    TypedArray = _GenericAlias(_TypedArray, (Any, DType))
+elif version.parse('1.22.0') <= version.parse(
+        np.__version__) < version.parse('1.23.0'):
     from numpy.typing import _GenericAlias
     TypedArray = _GenericAlias(_TypedArray, (Any, DType))
 else:
-    TypedArray = _TypedArray[Any, DType]
+    TypedArray = _TypedArray[Any, DType]
\ No newline at end of file
