@@ -59,7 +59,7 @@ def feature_confusion_matrix(
         Returns None if there are no annotations in ground_truth or prediction annotations
     """
     if has_no_matching_annotations(ground_truths, predictions):
-        return [0, int(len(predictions) > 0), 0, int(len(ground_truths) > 0)]
+        return [0, len(predictions), 0, len(ground_truths)]
     elif has_no_annotations(ground_truths, predictions):
         return None
     elif isinstance(predictions[0].value, Mask):
@@ -96,7 +96,7 @@ def classification_confusion_matrix(
     """
 
     if has_no_matching_annotations(ground_truths, predictions):
-        return [0, int(len(predictions) > 0), 0, int(len(ground_truths) > 0)]
+        return [0, len(predictions), 0, len(ground_truths)]
     elif has_no_annotations(
             ground_truths,
             predictions) or len(predictions) > 1 or len(ground_truths) > 1:
@@ -141,7 +141,7 @@ def vector_confusion_matrix(ground_truths: List[ObjectAnnotation],
          Returns None if there are no annotations in ground_truth or prediction annotations
     """
     if has_no_matching_annotations(ground_truths, predictions):
-        return [0, int(len(predictions) > 0), 0, int(len(ground_truths) > 0)]
+        return [0, len(predictions), 0, len(ground_truths)]
     elif has_no_annotations(ground_truths, predictions):
         return None
 
@@ -264,33 +264,14 @@ def mask_confusion_matrix(ground_truths: List[ObjectAnnotation],
         confusion matrix as a list: [TP,FP,TN,FN]
     """
     if has_no_matching_annotations(ground_truths, predictions):
-        return [0, int(len(predictions) > 0), 0, int(len(ground_truths) > 0)]
+        return [0, len(predictions), 0, len(ground_truths)]
     elif has_no_annotations(ground_truths, predictions):
         return None
 
-    if include_subclasses:
-        # This results in a faily drastically different value than without subclasses.
-        # If we have subclasses set to True, then this is object detection with masks
-        # Otherwise this will compute metrics on each pixel.
-        pairs = _get_mask_pairs(ground_truths, predictions)
-        return object_pair_confusion_matrix(
-            pairs, include_subclasses=include_subclasses, iou=iou)
-
-    prediction_np = np.max([pred.value.draw(color=1) for pred in predictions],
-                           axis=0)
-    ground_truth_np = np.max(
-        [ground_truth.value.draw(color=1) for ground_truth in ground_truths],
-        axis=0)
-    if prediction_np.shape != ground_truth_np.shape:
-        raise ValueError(
-            "Prediction and mask must have the same shape."
-            f" Found {prediction_np.shape}/{ground_truth_np.shape}.")
-
-    tp_mask = prediction_np == ground_truth_np == 1
-    fp_mask = (prediction_np == 1) & (ground_truth_np == 0)
-    fn_mask = (prediction_np == 0) & (ground_truth_np == 1)
-    tn_mask = prediction_np == ground_truth_np == 0
-    return [np.sum(tp_mask), np.sum(fp_mask), np.sum(fn_mask), np.sum(tn_mask)]
+    pairs = _get_mask_pairs(ground_truths, predictions)
+    return object_pair_confusion_matrix(pairs,
+                                        include_subclasses=include_subclasses,
+                                        iou=iou)
 
 
 def ner_confusion_matrix(ground_truths: List[ObjectAnnotation],
@@ -306,7 +287,7 @@ def ner_confusion_matrix(ground_truths: List[ObjectAnnotation],
         confusion matrix as a list: [TP,FP,TN,FN]
     """
     if has_no_matching_annotations(ground_truths, predictions):
-        return [0, int(len(predictions) > 0), 0, int(len(ground_truths) > 0)]
+        return [0, len(predictions), 0, len(ground_truths)]
     elif has_no_annotations(ground_truths, predictions):
         return None
     pairs = _get_ner_pairs(ground_truths, predictions)
