@@ -110,7 +110,7 @@ class Review(BaseModel):
     id: str
     created_at: str
     created_by: str
-    label_id: str
+    label_id: Optional[str] = None
 
     class Config:
         alias_generator = camel_case
@@ -238,7 +238,8 @@ class LBV1Label(BaseModel):
         ]) > 0
 
     def _row_contains(self, substrs) -> bool:
-        return any([substr in self.row_data for substr in substrs])
+        lower_row_data = self.row_data.lower()
+        return any([substr in lower_row_data for substr in substrs])
 
     def _is_url(self) -> bool:
         return self.row_data.startswith(
@@ -246,4 +247,4 @@ class LBV1Label(BaseModel):
              "s3://")) or "tileLayerUrl" in self.row_data
 
     class Config:
-        allow_population_by_field_name = True
\ No newline at end of file
+        allow_population_by_field_name = True
