@@ -1,7 +1,8 @@
 from ast import Bytes
 from io import BytesIO
-from typing import Any, Dict, List, Tuple, Union
+from typing import Any, Dict, List, Tuple, Union, Optional
 import base64
+from labelbox.data.mixins import ConfidenceMixin
 import numpy as np
 
 from pydantic import BaseModel
@@ -24,7 +25,7 @@ class NDBaseObject(NDAnnotation):
 
 
 class VideoSupported(BaseModel):
-    #support for video for objects are per-frame basis
+    # support for video for objects are per-frame basis
     frame: int
 
 
@@ -40,17 +41,21 @@ class Bbox(BaseModel):
     width: float
 
 
-class NDPoint(NDBaseObject):
+class NDPoint(NDBaseObject, ConfidenceMixin):
     point: _Point
 
     def to_common(self) -> Point:
         return Point(x=self.point.x, y=self.point.y)
 
     @classmethod
-    def from_common(cls, point: Point,
-                    classifications: List[ClassificationAnnotation], name: str,
-                    feature_schema_id: Cuid, extra: Dict[str, Any],
-                    data: Union[ImageData, TextData]) -> "NDPoint":
+    def from_common(cls,
+                    point: Point,
+                    classifications: List[ClassificationAnnotation],
+                    name: str,
+                    feature_schema_id: Cuid,
+                    extra: Dict[str, Any],
+                    data: Union[ImageData, TextData],
+                    confidence: Optional[float] = None) -> "NDPoint":
         return cls(point={
             'x': point.x,
             'y': point.y
@@ -59,7 +64,8 @@ class NDPoint(NDBaseObject):
                    name=name,
                    schema_id=feature_schema_id,
                    uuid=extra.get('uuid'),
-                   classifications=classifications)
+                   classifications=classifications,
+                   confidence=confidence)
 
 
 class NDFramePoint(VideoSupported):
@@ -80,17 +86,21 @@ class NDFramePoint(VideoSupported):
         return cls(frame=frame, point=_Point(x=point.x, y=point.y))
 
 
-class NDLine(NDBaseObject):
+class NDLine(NDBaseObject, ConfidenceMixin):
     line: List[_Point]
 
     def to_common(self) -> Line:
         return Line(points=[Point(x=pt.x, y=pt.y) for pt in self.line])
 
     @classmethod
-    def from_common(cls, line: Line,
-                    classifications: List[ClassificationAnnotation], name: str,
-                    feature_schema_id: Cuid, extra: Dict[str, Any],
-                    data: Union[ImageData, TextData]) -> "NDLine":
+    def from_common(cls,
+                    line: Line,
+                    classifications: List[ClassificationAnnotation],
+                    name: str,
+                    feature_schema_id: Cuid,
+                    extra: Dict[str, Any],
+                    data: Union[ImageData, TextData],
+                    confidence: Optional[float] = None) -> "NDLine":
         return cls(line=[{
             'x': pt.x,
             'y': pt.y
@@ -99,7 +109,8 @@ class NDLine(NDBaseObject):
                    name=name,
                    schema_id=feature_schema_id,
                    uuid=extra.get('uuid'),
-                   classifications=classifications)
+                   classifications=classifications,
+                   confidence=confidence)
 
 
 class NDFrameLine(VideoSupported):
@@ -124,17 +135,21 @@ class NDFrameLine(VideoSupported):
                    } for pt in line.points])
 
 
-class NDPolygon(NDBaseObject):
+class NDPolygon(NDBaseObject, ConfidenceMixin):
     polygon: List[_Point]
 
     def to_common(self) -> Polygon:
         return Polygon(points=[Point(x=pt.x, y=pt.y) for pt in self.polygon])
 
     @classmethod
-    def from_common(cls, polygon: Polygon,
-                    classifications: List[ClassificationAnnotation], name: str,
-                    feature_schema_id: Cuid, extra: Dict[str, Any],
-                    data: Union[ImageData, TextData]) -> "NDPolygon":
+    def from_common(cls,
+                    polygon: Polygon,
+                    classifications: List[ClassificationAnnotation],
+                    name: str,
+                    feature_schema_id: Cuid,
+                    extra: Dict[str, Any],
+                    data: Union[ImageData, TextData],
+                    confidence: Optional[float] = None) -> "NDPolygon":
         return cls(polygon=[{
             'x': pt.x,
             'y': pt.y
@@ -143,10 +158,11 @@ class NDPolygon(NDBaseObject):
                    name=name,
                    schema_id=feature_schema_id,
                    uuid=extra.get('uuid'),
-                   classifications=classifications)
+                   classifications=classifications,
+                   confidence=confidence)
 
 
-class NDRectangle(NDBaseObject):
+class NDRectangle(NDBaseObject, ConfidenceMixin):
     bbox: Bbox
 
     def to_common(self) -> Rectangle:
@@ -155,10 +171,14 @@ class NDRectangle(NDBaseObject):
                                    y=self.bbox.top + self.bbox.height))
 
     @classmethod
-    def from_common(cls, rectangle: Rectangle,
-                    classifications: List[ClassificationAnnotation], name: str,
-                    feature_schema_id: Cuid, extra: Dict[str, Any],
-                    data: Union[ImageData, TextData]) -> "NDRectangle":
+    def from_common(cls,
+                    rectangle: Rectangle,
+                    classifications: List[ClassificationAnnotation],
+                    name: str,
+                    feature_schema_id: Cuid,
+                    extra: Dict[str, Any],
+                    data: Union[ImageData, TextData],
+                    confidence: Optional[float] = None) -> "NDRectangle":
         return cls(bbox=Bbox(top=rectangle.start.y,
                              left=rectangle.start.x,
                              height=rectangle.end.y - rectangle.start.y,
@@ -167,7 +187,10 @@ class NDRectangle(NDBaseObject):
                    name=name,
                    schema_id=feature_schema_id,
                    uuid=extra.get('uuid'),
-                   classifications=classifications)
+                   classifications=classifications,
+                   page=extra.get('page'),
+                   unit=extra.get('unit'),
+                   confidence=confidence)
 
 
 class NDFrameRectangle(VideoSupported):
@@ -272,7 +295,7 @@ class _PNGMask(BaseModel):
     png: str
 
 
-class NDMask(NDBaseObject):
+class NDMask(NDBaseObject, ConfidenceMixin):
     mask: Union[_URIMask, _PNGMask]
 
     def to_common(self) -> Mask:
@@ -290,10 +313,14 @@ class NDMask(NDBaseObject):
             return Mask(mask=MaskData.from_2D_arr(image), color=(1, 1, 1))
 
     @classmethod
-    def from_common(cls, mask: Mask,
-                    classifications: List[ClassificationAnnotation], name: str,
-                    feature_schema_id: Cuid, extra: Dict[str, Any],
-                    data: Union[ImageData, TextData]) -> "NDMask":
+    def from_common(cls,
+                    mask: Mask,
+                    classifications: List[ClassificationAnnotation],
+                    name: str,
+                    feature_schema_id: Cuid,
+                    extra: Dict[str, Any],
+                    data: Union[ImageData, TextData],
+                    confidence: Optional[float] = None) -> "NDMask":
 
         if mask.mask.url is not None:
             lbv1_mask = _URIMask(instanceURI=mask.mask.url, colorRGB=mask.color)
@@ -309,7 +336,8 @@ class NDMask(NDBaseObject):
                    name=name,
                    schema_id=feature_schema_id,
                    uuid=extra.get('uuid'),
-                   classifications=classifications)
+                   classifications=classifications,
+                   confidence=confidence)
 
 
 class Location(BaseModel):
@@ -317,17 +345,21 @@ class Location(BaseModel):
     end: int
 
 
-class NDTextEntity(NDBaseObject):
+class NDTextEntity(NDBaseObject, ConfidenceMixin):
     location: Location
 
     def to_common(self) -> TextEntity:
         return TextEntity(start=self.location.start, end=self.location.end)
 
     @classmethod
-    def from_common(cls, text_entity: TextEntity,
-                    classifications: List[ClassificationAnnotation], name: str,
-                    feature_schema_id: Cuid, extra: Dict[str, Any],
-                    data: Union[ImageData, TextData]) -> "NDTextEntity":
+    def from_common(cls,
+                    text_entity: TextEntity,
+                    classifications: List[ClassificationAnnotation],
+                    name: str,
+                    feature_schema_id: Cuid,
+                    extra: Dict[str, Any],
+                    data: Union[ImageData, TextData],
+                    confidence: Optional[float] = None) -> "NDTextEntity":
         return cls(location=Location(
             start=text_entity.start,
             end=text_entity.end,
@@ -336,7 +368,8 @@ class NDTextEntity(NDBaseObject):
                    name=name,
                    schema_id=feature_schema_id,
                    uuid=extra.get('uuid'),
-                   classifications=classifications)
+                   classifications=classifications,
+                   confidence=confidence)
 
 
 class NDObject:
@@ -348,11 +381,18 @@ class NDObject:
             NDSubclassification.to_common(annot)
             for annot in annotation.classifications
         ]
+        confidence = annotation.confidence if hasattr(annotation,
+                                                      'confidence') else None
         return ObjectAnnotation(value=common_annotation,
                                 name=annotation.name,
                                 feature_schema_id=annotation.schema_id,
                                 classifications=classifications,
-                                extra={'uuid': annotation.uuid})
+                                extra={
+                                    'uuid': annotation.uuid,
+                                    'page': annotation.page,
+                                    'unit': annotation.unit
+                                },
+                                confidence=confidence)
 
     @classmethod
     def from_common(
@@ -362,7 +402,7 @@ class NDObject:
     ) -> Union[NDLine, NDPoint, NDPolygon, NDRectangle, NDMask, NDTextEntity]:
         obj = cls.lookup_object(annotation)
 
-        #if it is video segments
+        # if it is video segments
         if (obj == NDSegments):
             return obj.from_common(
                 annotation,
@@ -375,9 +415,12 @@ class NDObject:
             NDSubclassification.from_common(annot)
             for annot in annotation.classifications
         ]
+        optional_kwargs = {}
+        if (annotation.confidence):
+            optional_kwargs['confidence'] = annotation.confidence
         return obj.from_common(annotation.value, subclasses, annotation.name,
                                annotation.feature_schema_id, annotation.extra,
-                               data)
+                               data, **optional_kwargs)
 
     @staticmethod
     def lookup_object(
