@@ -371,11 +371,15 @@ class Entity(metaclass=EntityMeta):
         LabelingFrontendOptions: Type[labelbox.LabelingFrontendOptions]
         Label: Type[labelbox.Label]
         MEAPredictionImport: Type[labelbox.MEAPredictionImport]
+        MALPredictionImport: Type[labelbox.MALPredictionImport]
         Invite: Type[labelbox.Invite]
         InviteLimit: Type[labelbox.InviteLimit]
         ProjectRole: Type[labelbox.ProjectRole]
         Project: Type[labelbox.Project]
         Batch: Type[labelbox.Batch]
+        CatalogSlice: Type[labelbox.CatalogSlice]
+        ModelSlice: Type[labelbox.ModelSlice]
+        TaskQueue: Type[labelbox.TaskQueue]
 
     @classmethod
     def _attributes_of_type(cls, attr_type):
@@ -387,7 +391,7 @@ class Entity(metaclass=EntityMeta):
 
     @classmethod
     def fields(cls):
-        """ Returns a generateor that yields all the Fields declared in a
+        """ Returns a generator that yields all the Fields declared in a
         concrete subclass.
         """
         for attr in cls._attributes_of_type(Field):
@@ -396,7 +400,7 @@ class Entity(metaclass=EntityMeta):
 
     @classmethod
     def relationships(cls):
-        """ Returns a generateor that yields all the Relationships declared in
+        """ Returns a generator that yields all the Relationships declared in
         a concrete subclass.
         """
         return cls._attributes_of_type(Relationship)
