@@ -3,18 +3,23 @@ import json
 import logging
 import os
 import time
-from typing import Any, Dict, List, BinaryIO
-from tqdm import tqdm  # type: ignore
+from typing import Any, BinaryIO, Dict, List, Union, TYPE_CHECKING, cast
 
 import backoff
 import ndjson
 import requests
+from tqdm import tqdm  # type: ignore
 
 import labelbox
 from labelbox.orm import query
 from labelbox.orm.db_object import DbObject
 from labelbox.orm.model import Field, Relationship
+from labelbox.schema.confidence_presence_checker import LabelsConfidencePresenceChecker
 from labelbox.schema.enums import AnnotationImportState
+from labelbox.schema.serialization import serialize_labels
+
+if TYPE_CHECKING:
+    from labelbox.types import Label
 
 NDJSON_MIME_TYPE = "application/x-ndjson"
 logger = logging.getLogger(__name__)
@@ -140,6 +145,22 @@ class AnnotationImport(DbObject):
         files = {file_name: file_data}
         return client.execute(data=data, files=files)
 
+    @classmethod
+    def _get_ndjson_from_objects(cls, objects: Union[List[Dict[str, Any]],
+                                                     List["Label"]],
+                                 object_name: str) -> BinaryIO:
+        if not isinstance(objects, list):
+            raise TypeError(
+                f"{object_name} must be in a form of list. Found {type(objects)}"
+            )
+
+        objects = serialize_labels(objects)
+        data_str = ndjson.dumps(objects)
+        if not data_str:
+            raise ValueError(f"{object_name} cannot be empty")
+
+        return data_str.encode('utf-8')
+
     def refresh(self) -> None:
         """Synchronizes values of all fields with the database.
         """
@@ -196,8 +217,10 @@ class MEAPredictionImport(AnnotationImport):
             raise ValueError(f"File {path} is not accessible")
 
     @classmethod
-    def create_from_objects(cls, client: "labelbox.Client", model_run_id: str,
-                            name, predictions) -> "MEAPredictionImport":
+    def create_from_objects(
+        cls, client: "labelbox.Client", model_run_id: str, name,
+        predictions: Union[List[Dict[str, Any]], List["Label"]]
+    ) -> "MEAPredictionImport":
         """
         Create an MEA prediction import job from an in memory dictionary
 
@@ -209,12 +232,10 @@ class MEAPredictionImport(AnnotationImport):
         Returns:
             MEAPredictionImport
         """
-        data_str = ndjson.dumps(predictions)
-        if not data_str:
-            raise ValueError('annotations cannot be empty')
-        data = data_str.encode('utf-8')
+        data = cls._get_ndjson_from_objects(predictions, 'annotations')
+
         return cls._create_mea_import_from_bytes(client, model_run_id, name,
-                                                 data, len(data))
+                                                 data, len(str(data)))
 
     @classmethod
     def create_from_url(cls, client: "labelbox.Client", model_run_id: str,
@@ -318,6 +339,88 @@ class MEAPredictionImport(AnnotationImport):
         return cls(client, res["createModelErrorAnalysisPredictionImport"])
 
 
+class MEAToMALPredictionImport(AnnotationImport):
+    project = Relationship.ToOne("Project", cache=True)
+
+    @property
+    def parent_id(self) -> str:
+        """
+        Identifier for this import. Used to refresh the status
+        """
+        return self.project().uid
+
+    @classmethod
+    def create_for_model_run_data_rows(cls, client: "labelbox.Client",
+                                       model_run_id: str,
+                                       data_row_ids: List[str], project_id: str,
+                                       name: str) -> "MEAToMALPredictionImport":
+        """
+        Create an MEA to MAL prediction import job from a list of data row ids of a specific model run
+
+        Args:
+            client: Labelbox Client for executing queries
+            data_row_ids: A list of data row ids
+            model_run_id: model run id
+        Returns:
+            MEAToMALPredictionImport
+        """
+        query_str = cls._get_model_run_data_rows_mutation()
+        return cls(
+            client,
+            client.execute(query_str,
+                           params={
+                               "dataRowIds": data_row_ids,
+                               "modelRunId": model_run_id,
+                               "projectId": project_id,
+                               "name": name
+                           })["createMalPredictionImportForModelRunDataRows"])
+
+    @classmethod
+    def from_name(cls,
+                  client: "labelbox.Client",
+                  project_id: str,
+                  name: str,
+                  as_json: bool = False) -> "MEAToMALPredictionImport":
+        """
+        Retrieves an MEA to MAL import job.
+
+        Args:
+            client: Labelbox Client for executing queries
+            project_id:  ID used for querying import jobs
+            name: Name of the import job.
+        Returns:
+            MALPredictionImport
+        """
+        query_str = """query getMEAToMALPredictionImportPyApi($projectId : ID!, $name: String!) {
+            meaToMalPredictionImport(
+                where: {projectId: $projectId, name: $name}){
+                    %s
+                }}""" % query.results_query_part(cls)
+        params = {
+            "projectId": project_id,
+            "name": name,
+        }
+        response = client.execute(query_str, params)
+        if response is None:
+            raise labelbox.exceptions.ResourceNotFoundError(
+                MALPredictionImport, params)
+        response = response["meaToMalPredictionImport"]
+        if as_json:
+            return response
+        return cls(client, response)
+
+    @classmethod
+    def _get_model_run_data_rows_mutation(cls) -> str:
+        return """mutation createMalPredictionImportForModelRunDataRowsPyApi($dataRowIds: [ID!]!, $name: String!, $modelRunId: ID!, $projectId:ID!) {
+            createMalPredictionImportForModelRunDataRows(data: {
+                name: $name
+                modelRunId: $modelRunId
+                dataRowIds: $dataRowIds
+                projectId: $projectId
+            }) {%s}
+        }""" % query.results_query_part(cls)
+
+
 class MALPredictionImport(AnnotationImport):
     project = Relationship.ToOne("Project", cache=True)
 
@@ -352,8 +455,9 @@ class MALPredictionImport(AnnotationImport):
 
     @classmethod
     def create_from_objects(
-            cls, client: "labelbox.Client", project_id: str, name: str,
-            predictions: List[Dict[str, Any]]) -> "MALPredictionImport":
+        cls, client: "labelbox.Client", project_id: str, name: str,
+        predictions: Union[List[Dict[str, Any]], List["Label"]]
+    ) -> "MALPredictionImport":
         """
         Create an MAL prediction import job from an in memory dictionary
 
@@ -365,12 +469,19 @@ class MALPredictionImport(AnnotationImport):
         Returns:
             MALPredictionImport
         """
-        data_str = ndjson.dumps(predictions)
-        if not data_str:
-            raise ValueError('annotations cannot be empty')
-        data = data_str.encode('utf-8')
+        data = cls._get_ndjson_from_objects(predictions, 'annotations')
+
+        if len(predictions) > 0 and isinstance(predictions[0], Dict):
+            predictions_dicts = cast(List[Dict[str, Any]], predictions)
+            has_confidence = LabelsConfidencePresenceChecker.check(
+                predictions_dicts)
+            if has_confidence:
+                logger.warning("""
+                Confidence scores are not supported in MAL Prediction Import.
+                Corresponding confidence score values will be ignored.
+                """)
         return cls._create_mal_import_from_bytes(client, project_id, name, data,
-                                                 len(data))
+                                                 len(str(data)))
 
     @classmethod
     def create_from_url(cls, client: "labelbox.Client", project_id: str,
@@ -503,9 +614,10 @@ class LabelImport(AnnotationImport):
             raise ValueError(f"File {path} is not accessible")
 
     @classmethod
-    def create_from_objects(cls, client: "labelbox.Client", project_id: str,
-                            name: str,
-                            labels: List[Dict[str, Any]]) -> "LabelImport":
+    def create_from_objects(
+            cls, client: "labelbox.Client", project_id: str, name: str,
+            labels: Union[List[Dict[str, Any]],
+                          List["Label"]]) -> "LabelImport":
         """
         Create a label import job from an in memory dictionary
 
@@ -517,12 +629,18 @@ class LabelImport(AnnotationImport):
         Returns:
             LabelImport
         """
-        data_str = ndjson.dumps(labels)
-        if not data_str:
-            raise ValueError('labels cannot be empty')
-        data = data_str.encode('utf-8')
+        data = cls._get_ndjson_from_objects(labels, 'labels')
+
+        if len(labels) > 0 and isinstance(labels[0], Dict):
+            label_dicts = cast(List[Dict[str, Any]], labels)
+            has_confidence = LabelsConfidencePresenceChecker.check(label_dicts)
+            if has_confidence:
+                logger.warning("""
+                Confidence scores are not supported in Label Import.
+                Corresponding confidence score values will be ignored.
+                """)
         return cls._create_label_import_from_bytes(client, project_id, name,
-                                                   data, len(data))
+                                                   data, len(str(data)))
 
     @classmethod
     def create_from_url(cls, client: "labelbox.Client", project_id: str,
