@@ -12,12 +12,18 @@ from labelbox.pagination import PaginatedCollection
 from labelbox.orm.query import results_query_part
 from labelbox.orm.model import Field, Relationship, Entity
 from labelbox.orm.db_object import DbObject, experimental
+from labelbox.schema.export_params import ModelRunExportParams
+from labelbox.schema.task import Task
+from labelbox.schema.user import User
 
 if TYPE_CHECKING:
     from labelbox import MEAPredictionImport
+    from labelbox.types import Label
 
 logger = logging.getLogger(__name__)
 
+DATAROWS_IMPORT_LIMIT = 25000
+
 
 class DataSplit(Enum):
     TRAINING = "TRAINING"
@@ -41,22 +47,43 @@ class ModelRun(DbObject):
         COMPLETE = "COMPLETE"
         FAILED = "FAILED"
 
-    def upsert_labels(self, label_ids, timeout_seconds=60):
+    def upsert_labels(self,
+                      label_ids: Optional[List[str]] = None,
+                      project_id: Optional[str] = None,
+                      timeout_seconds=3600):
         """ Adds data rows and labels to a Model Run
         Args:
             label_ids (list): label ids to insert
+            project_id (string): project uuid, all project labels will be uploaded
+                Either label_ids OR project_id is required but NOT both
             timeout_seconds (float): Max waiting time, in seconds.
         Returns:
             ID of newly generated async task
+
         """
 
-        if len(label_ids) < 1:
-            raise ValueError("Must provide at least one label id")
+        use_label_ids = label_ids is not None and len(label_ids) > 0
+        use_project_id = project_id is not None
 
+        if not use_label_ids and not use_project_id:
+            raise ValueError(
+                "Must provide at least one label id or a project id")
+
+        if use_label_ids and use_project_id:
+            raise ValueError("Must only one of label ids, project id")
+
+        if use_label_ids:
+            return self._upsert_labels_by_label_ids(label_ids, timeout_seconds)
+        else:  # use_project_id
+            return self._upsert_labels_by_project_id(project_id,
+                                                     timeout_seconds)
+
+    def _upsert_labels_by_label_ids(self, label_ids: List[str],
+                                    timeout_seconds: int):
         mutation_name = 'createMEAModelRunLabelRegistrationTask'
         create_task_query_str = """mutation createMEAModelRunLabelRegistrationTaskPyApi($modelRunId: ID!, $labelIds : [ID!]!) {
-          %s(where : { id : $modelRunId}, data : {labelIds: $labelIds})}
-          """ % (mutation_name)
+        %s(where : { id : $modelRunId}, data : {labelIds: $labelIds})}
+        """ % (mutation_name)
 
         res = self.client.execute(create_task_query_str, {
             'modelRunId': self.uid,
@@ -74,27 +101,53 @@ class ModelRun(DbObject):
             }})['MEALabelRegistrationTaskStatus'],
                                      timeout_seconds=timeout_seconds)
 
-    def upsert_data_rows(self, data_row_ids, timeout_seconds=60):
+    def _upsert_labels_by_project_id(self, project_id: str,
+                                     timeout_seconds: int):
+        mutation_name = 'createMEAModelRunProjectLabelRegistrationTask'
+        create_task_query_str = """mutation createMEAModelRunProjectLabelRegistrationTaskPyApi($modelRunId: ID!, $projectId : ID!) {
+        %s(where : { modelRunId : $modelRunId, projectId: $projectId})}
+        """ % (mutation_name)
+
+        res = self.client.execute(create_task_query_str, {
+            'modelRunId': self.uid,
+            'projectId': project_id
+        })
+        task_id = res[mutation_name]
+
+        status_query_str = """query MEALabelRegistrationTaskStatusPyApi($where: WhereUniqueIdInput!){
+            MEALabelRegistrationTaskStatus(where: $where) {status errorMessage}
+        }
+        """
+        return self._wait_until_done(lambda: self.client.execute(
+            status_query_str, {'where': {
+                'id': task_id
+            }})['MEALabelRegistrationTaskStatus'],
+                                     timeout_seconds=timeout_seconds)
+
+    def upsert_data_rows(self,
+                         data_row_ids=None,
+                         global_keys=None,
+                         timeout_seconds=3600):
         """ Adds data rows to a Model Run without any associated labels
         Args:
-            data_row_ids (list): data row ids to add to mea
+            data_row_ids (list): data row ids to add to model run
+            global_keys (list): global keys for data rows to add to model run
             timeout_seconds (float): Max waiting time, in seconds.
         Returns:
             ID of newly generated async task
         """
 
-        if len(data_row_ids) < 1:
-            raise ValueError("Must provide at least one data row id")
-
         mutation_name = 'createMEAModelRunDataRowRegistrationTask'
-        create_task_query_str = """mutation createMEAModelRunDataRowRegistrationTaskPyApi($modelRunId: ID!, $dataRowIds : [ID!]!) {
-          %s(where : { id : $modelRunId}, data : {dataRowIds: $dataRowIds})}
+        create_task_query_str = """mutation createMEAModelRunDataRowRegistrationTaskPyApi($modelRunId: ID!, $dataRowIds: [ID!], $globalKeys: [ID!]) {
+          %s(where : { id : $modelRunId}, data : {dataRowIds: $dataRowIds, globalKeys: $globalKeys})}
           """ % (mutation_name)
 
-        res = self.client.execute(create_task_query_str, {
-            'modelRunId': self.uid,
-            'dataRowIds': data_row_ids
-        })
+        res = self.client.execute(
+            create_task_query_str, {
+                'modelRunId': self.uid,
+                'dataRowIds': data_row_ids,
+                'globalKeys': global_keys
+            })
         task_id = res[mutation_name]
 
         status_query_str = """query MEADataRowRegistrationTaskStatusPyApi($where: WhereUniqueIdInput!){
@@ -123,10 +176,94 @@ class ModelRun(DbObject):
                 )
             time.sleep(sleep_time)
 
-    def add_predictions(
+    def upsert_predictions_and_send_to_project(
         self,
         name: str,
         predictions: Union[str, Path, Iterable[Dict]],
+        project_id: str,
+        priority: Optional[int] = 5,
+    ) -> 'MEAPredictionImport':  # type: ignore
+        """
+        Provides a convenient way to execute the following steps in a single function call:
+            1. Upload predictions to a Model
+            2. Create a batch from data rows that had predictions assocated with them
+            3. Attach the batch to a project
+            4. Add those same predictions to the project as MAL annotations
+
+        Note that partial successes are possible.
+        If it is important that all stages are successful then check the status of each individual task
+        with task.errors. E.g.
+
+        >>>    mea_import_job, batch, mal_import_job = upsert_predictions_and_send_to_project(name, predictions, project_id)
+        >>>    # handle mea import job successfully created (check for job failure or partial failures)
+        >>>    print(mea_import_job.status, mea_import_job.errors)
+        >>>    if batch is None:
+        >>>        # Handle batch creation failure
+        >>>    if mal_import_job is None:
+        >>>        # Handle mal_import_job creation failure
+        >>>    else:
+        >>>        # handle mal import job successfully created (check for job failure or partial failures)
+        >>>        print(mal_import_job.status, mal_import_job.errors)
+
+
+        Args:
+            name (str): name of the AnnotationImport job as well as the name of the batch import
+            predictions (Iterable):
+                iterable of annotation rows
+            project_id (str): id of the project to import into
+            priority (int): priority of the job
+        Returns:
+            Tuple[MEAPredictionImport, Batch, MEAToMALPredictionImport]
+            If any of these steps fail the return value will be None.
+
+        """
+        kwargs = dict(client=self.client, model_run_id=self.uid, name=name)
+        project = self.client.get_project(project_id)
+        import_job = self.add_predictions(name, predictions)
+        prediction_statuses = import_job.statuses
+        mea_to_mal_data_rows = list(
+            set([
+                row['dataRow']['id']
+                for row in prediction_statuses
+                if row['status'] == 'SUCCESS'
+            ]))
+
+        if not mea_to_mal_data_rows:
+            # 0 successful model predictions imported
+            return import_job, None, None
+
+        elif len(mea_to_mal_data_rows) >= DATAROWS_IMPORT_LIMIT:
+            mea_to_mal_data_rows = mea_to_mal_data_rows[:DATAROWS_IMPORT_LIMIT]
+            logger.warning(
+                f"Exeeded max data row limit {len(mea_to_mal_data_rows)}, trimmed down to {DATAROWS_IMPORT_LIMIT} data rows."
+            )
+
+        try:
+            batch = project.create_batch(name, mea_to_mal_data_rows, priority)
+        except Exception as e:
+            logger.warning(f"Failed to create batch. Messsage : {e}.")
+            # Unable to create batch
+            return import_job, None, None
+
+        try:
+            mal_prediction_import = Entity.MEAToMALPredictionImport.create_for_model_run_data_rows(
+                data_row_ids=mea_to_mal_data_rows,
+                project_id=project_id,
+                **kwargs)
+            mal_prediction_import.wait_until_done()
+        except Exception as e:
+            logger.warning(
+                f"Failed to create MEA to MAL prediction import. Message : {e}."
+            )
+            # Unable to create mea to mal prediction import
+            return import_job, batch, None
+
+        return import_job, batch, mal_prediction_import
+
+    def add_predictions(
+        self,
+        name: str,
+        predictions: Union[str, Path, Iterable[Dict], Iterable["Label"]],
     ) -> 'MEAPredictionImport':  # type: ignore
         """ Uploads predictions to a new Editor project.
         Args:
@@ -198,14 +335,15 @@ class ModelRun(DbObject):
 
     @experimental
     def assign_data_rows_to_split(self,
-                                  data_row_ids: List[str],
-                                  split: Union[DataSplit, str],
+                                  data_row_ids: List[str] = None,
+                                  split: Union[DataSplit, str] = None,
+                                  global_keys: List[str] = None,
                                   timeout_seconds=120):
 
         split_value = split.value if isinstance(split, DataSplit) else split
         valid_splits = DataSplit._member_names_
 
-        if split_value not in valid_splits:
+        if split_value is None or split_value not in valid_splits:
             raise ValueError(
                 f"`split` must be one of : `{valid_splits}`. Found : `{split}`")
 
@@ -217,7 +355,8 @@ class ModelRun(DbObject):
                 'data': {
                     'assignments': [{
                         'split': split_value,
-                        'dataRowIds': data_row_ids
+                        'dataRowIds': data_row_ids,
+                        'globalKeys': global_keys,
                     }]
                 }
             },
@@ -264,11 +403,11 @@ class ModelRun(DbObject):
 
     @experimental
     def update_config(self, config: Dict[str, Any]) -> Dict[str, Any]:
-        """ 
+        """
          Updates the Model Run's training metadata config
-         Args: 
+         Args:
              config (dict): A dictionary of keys and values
-         Returns: 
+         Returns:
              Model Run id and updated training metadata
          """
         data: Dict[str, Any] = {'config': config}
@@ -285,9 +424,9 @@ class ModelRun(DbObject):
 
     @experimental
     def reset_config(self) -> Dict[str, Any]:
-        """ 
+        """
          Resets Model Run's training metadata config
-         Returns: 
+         Returns:
              Model Run id and reset training metadata
          """
         res = self.client.execute(
@@ -300,10 +439,10 @@ class ModelRun(DbObject):
 
     @experimental
     def get_config(self) -> Dict[str, Any]:
-        """ 
-         Gets Model Run's training metadata 
-         Returns: 
-             training metadata as a dictionary 
+        """
+         Gets Model Run's training metadata
+         Returns:
+             training metadata as a dictionary
          """
         res = self.client.execute("""query ModelRunPyApi($modelRunId: ID!){
                 modelRun(where: {id : $modelRunId}){trainingMetadata}
@@ -360,6 +499,59 @@ class ModelRun(DbObject):
                          self.uid)
             time.sleep(sleep_time)
 
+    """
+    Creates a model run export task with the given params and returns the task.
+    
+    >>>    export_task = export_v2("my_export_task", params={"media_attributes": True})
+    
+    """
+
+    def export_v2(self,
+                  task_name: Optional[str] = None,
+                  params: Optional[ModelRunExportParams] = None) -> Task:
+        mutation_name = "exportDataRowsInModelRun"
+        create_task_query_str = """mutation exportDataRowsInModelRunPyApi($input: ExportDataRowsInModelRunInput!){
+          %s(input: $input) {taskId} }
+          """ % (mutation_name)
+
+        _params = params or ModelRunExportParams()
+
+        queryParams = {
+            "input": {
+                "taskName": task_name,
+                "filters": {
+                    "modelRunId": self.uid
+                },
+                "params": {
+                    "mediaTypeOverride":
+                        _params.get('media_type_override', None),
+                    "includeAttachments":
+                        _params.get('attachments', False),
+                    "includeMetadata":
+                        _params.get('metadata_fields', False),
+                    "includeDataRowDetails":
+                        _params.get('data_row_details', False),
+                },
+            }
+        }
+        res = self.client.execute(
+            create_task_query_str,
+            queryParams,
+        )
+        res = res[mutation_name]
+        task_id = res["taskId"]
+        user: User = self.client.get_user()
+        tasks: List[Task] = list(
+            user.created_tasks(where=Entity.Task.uid == task_id))
+        # Cache user in a private variable as the relationship can't be
+        # resolved due to server-side limitations (see Task.created_by)
+        # for more info.
+        if len(tasks) != 1:
+            raise ResourceNotFoundError(Entity.Task, task_id)
+        task: Task = tasks[0]
+        task._user = user
+        return task
+
 
 class ModelRunDataRow(DbObject):
     label_id = Field.String("label_id")
