@@ -1,25 +1,33 @@
-from enum import Enum
 import json
 import logging
 import time
-import warnings
 from collections import namedtuple
 from datetime import datetime, timezone
 from pathlib import Path
-from typing import TYPE_CHECKING, Dict, Union, Iterable, List, Optional, Any
+from typing import TYPE_CHECKING, Any, Collection, Dict, Iterable, List, Optional, Union
 from urllib.parse import urlparse
 
 import ndjson
 import requests
 
 from labelbox import utils
-from labelbox.exceptions import InvalidQueryError, LabelboxError
+from labelbox.exceptions import (InvalidQueryError, LabelboxError,
+                                 ProcessingWaitTimeout, ResourceConflict,
+                                 ResourceNotFoundError)
 from labelbox.orm import query
-from labelbox.orm.db_object import DbObject, Updateable, Deletable
+from labelbox.orm.db_object import DbObject, Deletable, Updateable
 from labelbox.orm.model import Entity, Field, Relationship
 from labelbox.pagination import PaginatedCollection
-from labelbox.schema.resource_tag import ResourceTag
+from labelbox.schema.consensus_settings import ConsensusSettings
+from labelbox.schema.data_row import DataRow
+from labelbox.schema.export_filters import ProjectExportFilters
+from labelbox.schema.export_params import ProjectExportParams
 from labelbox.schema.media_type import MediaType
+from labelbox.schema.queue_mode import QueueMode
+from labelbox.schema.resource_tag import ResourceTag
+from labelbox.schema.task import Task
+from labelbox.schema.user import User
+from labelbox.schema.task_queue import TaskQueue
 
 if TYPE_CHECKING:
     from labelbox import BulkImportRequest
@@ -39,6 +47,20 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
+def _validate_datetime(string_date: str) -> bool:
+    """helper function validate that datetime is as follows: YYYY-MM-DD for the export"""
+    if string_date:
+        for fmt in ("%Y-%m-%d", "%Y-%m-%d %H:%M:%S"):
+            try:
+                datetime.strptime(string_date, fmt)
+                return True
+            except ValueError:
+                pass
+        raise ValueError(f"""Incorrect format for: {string_date}.
+        Format must be \"YYYY-MM-DD\" or \"YYYY-MM-DD hh:mm:ss\"""")
+    return True
+
+
 class Project(DbObject, Updateable, Deletable):
     """ A Project is a container that includes a labeling frontend, an ontology,
     datasets and labels.
@@ -50,6 +72,7 @@ class Project(DbObject, Updateable, Deletable):
         created_at (datetime)
         setup_complete (datetime)
         last_activity_time (datetime)
+        queue_mode (string)
         auto_audit_number_of_labels (int)
         auto_audit_percentage (float)
 
@@ -62,6 +85,7 @@ class Project(DbObject, Updateable, Deletable):
         webhooks (Relationship): `ToMany` relationship to Webhook
         benchmarks (Relationship): `ToMany` relationship to Benchmark
         ontology (Relationship): `ToOne` relationship to Ontology
+        task_queues (Relationship): `ToMany` relationship to TaskQueue
     """
 
     name = Field.String("name")
@@ -70,6 +94,7 @@ class Project(DbObject, Updateable, Deletable):
     created_at = Field.DateTime("created_at")
     setup_complete = Field.DateTime("setup_complete")
     last_activity_time = Field.DateTime("last_activity_time")
+    queue_mode = Field.Enum(QueueMode, "queue_mode")
     auto_audit_number_of_labels = Field.Int("auto_audit_number_of_labels")
     auto_audit_percentage = Field.Float("auto_audit_percentage")
     # Bind data_type and allowedMediaTYpe using the GraphQL type MediaType
@@ -88,9 +113,8 @@ class Project(DbObject, Updateable, Deletable):
     benchmarks = Relationship.ToMany("Benchmark", False)
     ontology = Relationship.ToOne("Ontology", True)
 
-    class QueueMode(Enum):
-        Batch = "Batch"
-        Dataset = "Dataset"
+    #
+    _wait_processing_max_seconds = 3600
 
     def update(self, **kwargs):
         """ Updates this project with the specified attributes
@@ -98,7 +122,9 @@ class Project(DbObject, Updateable, Deletable):
         Args:
             kwargs: a dictionary containing attributes to be upserted
 
-        Note that the quality setting cannot be changed after a project has been created. The quality mode
+        Note that the queue_mode cannot be changed after a project has been created.
+
+        Additionally, the quality setting cannot be changed after a project has been created. The quality mode
             for a project is inferred through the following attributes:
             Benchmark:
                 auto_audit_number_of_labels = 1
@@ -109,9 +135,6 @@ class Project(DbObject, Updateable, Deletable):
             Attempting to switch between benchmark and consensus modes is an invalid operation and will result
             in an error.
         """
-        mode: Optional[Project.QueueMode] = kwargs.pop("queue_mode", None)
-        if mode:
-            self._update_queue_mode(mode)
 
         media_type = kwargs.get("media_type")
         if media_type:
@@ -260,16 +283,21 @@ class Project(DbObject, Updateable, Deletable):
         json_data = self.export_labels(download=True,
                                        timeout_seconds=timeout_seconds,
                                        **kwargs)
+
         # assert that the instance this would fail is only if timeout runs out
         assert isinstance(
             json_data,
             List), "Unable to successfully get labels. Please try again"
+
         if json_data is None:
             raise TimeoutError(
                 f"Unable to download labels in {timeout_seconds} seconds."
                 "Please try again or contact support if the issue persists.")
+
         is_video = [
-            'frames' in row['Label'] for row in json_data if row['Label']
+            "frames" in row["Label"]
+            for row in json_data
+            if row["Label"] and not row["Skipped"]
         ]
 
         if len(is_video) and not all(is_video) and any(is_video):
@@ -278,7 +306,14 @@ class Project(DbObject, Updateable, Deletable):
                 "Use project.export_labels() to export projects with mixed data types. "
             )
         if len(is_video) and all(is_video):
+            # Filter skipped labels to avoid inference errors
+            json_data = [
+                label for label in self.export_labels(download=True)
+                if not label["Skipped"]
+            ]
+
             return LBV1Converter.deserialize_video(json_data, self.client)
+
         return LBV1Converter.deserialize(json_data)
 
     def export_labels(self,
@@ -293,8 +328,12 @@ class Project(DbObject, Updateable, Deletable):
         Args:
             download (bool): Returns the url if False
             timeout_seconds (float): Max waiting time, in seconds.
-            start (str): Earliest date for labels, formatted "YYYY-MM-DD"
-            end (str): Latest date for labels, formatted "YYYY-MM-DD"
+            start (str): Earliest date for labels, formatted "YYYY-MM-DD" or "YYYY-MM-DD hh:mm:ss"
+            end (str): Latest date for labels, formatted "YYYY-MM-DD" or "YYYY-MM-DD hh:mm:ss"
+            last_activity_start (str): Will include all labels that have had any updates to
+              data rows, issues, comments, metadata, or reviews since this timestamp.
+              formatted "YYYY-MM-DD" or "YYYY-MM-DD hh:mm:ss"
+
         Returns:
             URL of the data file with this Project's labels. If the server didn't
             generate during the `timeout_seconds` period, None is returned.
@@ -313,16 +352,6 @@ class Project(DbObject, Updateable, Deletable):
                 if dictionary.get(c)
             ])
 
-        def _validate_datetime(string_date: str) -> bool:
-            """helper function validate that datetime is as follows: YYYY-MM-DD for the export"""
-            if string_date:
-                try:
-                    datetime.strptime(string_date, "%Y-%m-%d")
-                except ValueError:
-                    raise ValueError(f"""Incorrect format for: {string_date}.
-                    Format must be \"YYYY-MM-DD\"""")
-            return True
-
         sleep_time = 2
         id_param = "projectId"
         filter_param = ""
@@ -337,6 +366,12 @@ class Project(DbObject, Updateable, Deletable):
             filter_param_dict["labelCreatedAt"] = "{%s}" % _string_from_dict(
                 created_at_dict, value_with_quotes=True)
 
+        if "last_activity_start" in kwargs:
+            last_activity_start = kwargs['last_activity_start']
+            _validate_datetime(last_activity_start)
+            filter_param_dict["lastActivityAt"] = "{%s}" % _string_from_dict(
+                {"start": last_activity_start}, value_with_quotes=True)
+
         if filter_param_dict:
             filter_param = """, filters: {%s }""" % (_string_from_dict(
                 filter_param_dict, value_with_quotes=False))
@@ -346,6 +381,7 @@ class Project(DbObject, Updateable, Deletable):
         """ % (id_param, id_param, filter_param)
 
         start_time = time.time()
+
         while True:
             res = self.client.execute(query_str, {id_param: self.uid})
             res = res["exportLabels"]
@@ -366,6 +402,182 @@ class Project(DbObject, Updateable, Deletable):
                          self.uid)
             time.sleep(sleep_time)
 
+    def export_v2(self,
+                  task_name: Optional[str] = None,
+                  filters: Optional[ProjectExportFilters] = None,
+                  params: Optional[ProjectExportParams] = None) -> Task:
+        """
+        Creates a project run export task with the given params and returns the task.
+
+        For more information visit: https://docs.labelbox.com/docs/exports-v2#export-from-a-project-python-sdk
+        
+        >>>     task = project.export_v2(
+        >>>         filters={
+        >>>             "last_activity_at": ["2000-01-01 00:00:00", "2050-01-01 00:00:00"],
+        >>>             "label_created_at": ["2000-01-01 00:00:00", "2050-01-01 00:00:00"]
+        >>>         },
+        >>>         params={
+        >>>             "include_performance_details": False,
+        >>>             "include_labels": True
+        >>>         })
+        >>>     task.wait_till_done()
+        >>>     task.result
+        """
+
+        _params = params or ProjectExportParams({
+            "attachments": False,
+            "metadata_fields": False,
+            "data_row_details": False,
+            "project_details": False,
+            "performance_details": False,
+            "label_details": False,
+            "media_type_override": None
+        })
+
+        _filters = filters or ProjectExportFilters({
+            "last_activity_at": None,
+            "label_created_at": None
+        })
+
+        def _get_timezone() -> str:
+            timezone_query_str = """query CurrentUserPyApi { user { timezone } }"""
+            tz_res = self.client.execute(timezone_query_str)
+            return tz_res["user"]["timezone"] or "UTC"
+
+        timezone: Optional[str] = None
+
+        mutation_name = "exportDataRowsInProject"
+        create_task_query_str = """mutation exportDataRowsInProjectPyApi($input: ExportDataRowsInProjectInput!){
+          %s(input: $input) {taskId} }
+          """ % (mutation_name)
+
+        search_query: List[Dict[str, Collection[str]]] = []
+        media_type_override = _params.get('media_type_override', None)
+        query_params = {
+            "input": {
+                "taskName": task_name,
+                "filters": {
+                    "projectId": self.uid,
+                    "searchQuery": {
+                        "scope": None,
+                        "query": search_query
+                    }
+                },
+                "params": {
+                    "mediaTypeOverride":
+                        media_type_override.value
+                        if media_type_override is not None else None,
+                    "includeAttachments":
+                        _params.get('attachments', False),
+                    "includeMetadata":
+                        _params.get('metadata_fields', False),
+                    "includeDataRowDetails":
+                        _params.get('data_row_details', False),
+                    "includeProjectDetails":
+                        _params.get('project_details', False),
+                    "includePerformanceDetails":
+                        _params.get('performance_details', False),
+                    "includeLabelDetails":
+                        _params.get('label_details', False)
+                },
+            }
+        }
+
+        if "last_activity_at" in _filters and _filters[
+                'last_activity_at'] is not None:
+            if timezone is None:
+                timezone = _get_timezone()
+            values = _filters['last_activity_at']
+            start, end = values
+            if (start is not None and end is not None):
+                [_validate_datetime(date) for date in values]
+                search_query.append({
+                    "type": "data_row_last_activity_at",
+                    "value": {
+                        "operator": "BETWEEN",
+                        "timezone": timezone,
+                        "value": {
+                            "min": start,
+                            "max": end
+                        }
+                    }
+                })
+            elif (start is not None):
+                _validate_datetime(start)
+                search_query.append({
+                    "type": "data_row_last_activity_at",
+                    "value": {
+                        "operator": "GREATER_THAN_OR_EQUAL",
+                        "timezone": timezone,
+                        "value": start
+                    }
+                })
+            elif (end is not None):
+                _validate_datetime(end)
+                search_query.append({
+                    "type": "data_row_last_activity_at",
+                    "value": {
+                        "operator": "LESS_THAN_OR_EQUAL",
+                        "timezone": timezone,
+                        "value": end
+                    }
+                })
+
+        if "label_created_at" in _filters and _filters[
+                "label_created_at"] is not None:
+            if timezone is None:
+                timezone = _get_timezone()
+            values = _filters['label_created_at']
+            start, end = values
+            if (start is not None and end is not None):
+                [_validate_datetime(date) for date in values]
+                search_query.append({
+                    "type": "labeled_at",
+                    "value": {
+                        "operator": "BETWEEN",
+                        "value": {
+                            "min": start,
+                            "max": end
+                        }
+                    }
+                })
+            elif (start is not None):
+                _validate_datetime(start)
+                search_query.append({
+                    "type": "labeled_at",
+                    "value": {
+                        "operator": "GREATER_THAN_OR_EQUAL",
+                        "value": start
+                    }
+                })
+            elif (end is not None):
+                _validate_datetime(end)
+                search_query.append({
+                    "type": "labeled_at",
+                    "value": {
+                        "operator": "LESS_THAN_OR_EQUAL",
+                        "value": end
+                    }
+                })
+
+        res = self.client.execute(
+            create_task_query_str,
+            query_params,
+        )
+        res = res[mutation_name]
+        task_id = res["taskId"]
+        user: User = self.client.get_user()
+        tasks: List[Task] = list(
+            user.created_tasks(where=Entity.Task.uid == task_id))
+        # Cache user in a private variable as the relationship can't be
+        # resolved due to server-side limitations (see Task.created_by)
+        # for more info.
+        if len(tasks) != 1:
+            raise ResourceNotFoundError(Entity.Task, task_id)
+        task: Task = tasks[0]
+        task._user = user
+        return task
+
     def export_issues(self, status=None) -> str:
         """ Calls the server-side Issues exporting that
         returns the URL to that payload.
@@ -507,6 +719,9 @@ class Project(DbObject, Updateable, Deletable):
         Args:
             ontology (Ontology): The ontology to attach to the project
         """
+        if self.labeling_frontend() is not None:
+            raise ResourceConflict("Editor is already set up.")
+
         labeling_frontend = next(
             self.client.get_labeling_frontends(
                 where=Entity.LabelingFrontend.name == "Editor"))
@@ -546,6 +761,9 @@ class Project(DbObject, Updateable, Deletable):
                 to `str` using `json.dumps`.
         """
 
+        if self.labeling_frontend() is not None:
+            raise ResourceConflict("Editor is already set up.")
+
         if not isinstance(labeling_frontend_options, str):
             labeling_frontend_options = json.dumps(labeling_frontend_options)
 
@@ -562,71 +780,215 @@ class Project(DbObject, Updateable, Deletable):
         timestamp = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%SZ")
         self.update(setup_complete=timestamp)
 
-    def create_batch(self, name: str, data_rows: List[str], priority: int = 5):
-        """Create a new batch for a project. Batches is in Beta and subject to change
+    def create_batch(
+        self,
+        name: str,
+        data_rows: Optional[List[Union[str, DataRow]]] = None,
+        priority: int = 5,
+        consensus_settings: Optional[Dict[str, float]] = None,
+        global_keys: Optional[List[str]] = None,
+    ):
+        """Create a new batch for a project. One of `global_keys` or `data_rows` must be provided but not both.
 
         Args:
             name: a name for the batch, must be unique within a project
-            data_rows: Either a list of `DataRows` or Data Row ids
+            data_rows: Either a list of `DataRows` or Data Row ids. 
+            global_keys: global keys for data rows to add to the batch. 
             priority: An optional priority for the Data Rows in the Batch. 1 highest -> 5 lowest
-
+            consensus_settings: An optional dictionary with consensus settings: {'number_of_labels': 3, 'coverage_percentage': 0.1}
         """
 
         # @TODO: make this automatic?
-        if self.queue_mode() != Project.QueueMode.Batch:
+        if self.queue_mode != QueueMode.Batch:
             raise ValueError("Project must be in batch mode")
 
         dr_ids = []
-        for dr in data_rows:
-            if isinstance(dr, Entity.DataRow):
-                dr_ids.append(dr.uid)
-            elif isinstance(dr, str):
-                dr_ids.append(dr)
-            else:
-                raise ValueError("You can DataRow ids or DataRow objects")
+        if data_rows is not None:
+            for dr in data_rows:
+                if isinstance(dr, Entity.DataRow):
+                    dr_ids.append(dr.uid)
+                elif isinstance(dr, str):
+                    dr_ids.append(dr)
+                else:
+                    raise ValueError(
+                        "`data_rows` must be DataRow ids or DataRow objects")
 
-        if len(dr_ids) > 25_000:
+        if data_rows is not None:
+            row_count = len(dr_ids)
+        elif global_keys is not None:
+            row_count = len(global_keys)
+        else:
+            row_count = 0
+
+        if row_count > 100_000:
             raise ValueError(
                 f"Batch exceeds max size, break into smaller batches")
-        if not len(dr_ids):
+        if not row_count:
             raise ValueError("You need at least one data row in a batch")
 
-        method = 'createBatch'
-        query_str = """mutation %sPyApi($projectId: ID!, $batchInput: CreateBatchInput!) {
-              project(where: {id: $projectId}) {
-                %s(input: $batchInput) {
-                  %s
-                }
-              }
-            }
-        """ % (method, method, query.results_query_part(Entity.Batch))
+        self._wait_until_data_rows_are_processed(
+            dr_ids, global_keys, self._wait_processing_max_seconds)
 
+        if consensus_settings:
+            consensus_settings = ConsensusSettings(**consensus_settings).dict(
+                by_alias=True)
+
+        if len(dr_ids) >= 10_000:
+            return self._create_batch_async(name, dr_ids, global_keys, priority,
+                                            consensus_settings)
+        else:
+            return self._create_batch_sync(name, dr_ids, global_keys, priority,
+                                           consensus_settings)
+
+    def _create_batch_sync(self, name, dr_ids, global_keys, priority,
+                           consensus_settings):
+        method = 'createBatchV2'
+        query_str = """mutation %sPyApi($projectId: ID!, $batchInput: CreateBatchInput!) {
+                  project(where: {id: $projectId}) {
+                    %s(input: $batchInput) {
+                        batch {
+                            %s
+                        }
+                        failedDataRowIds
+                    }
+                  }
+                }
+            """ % (method, method, query.results_query_part(Entity.Batch))
         params = {
             "projectId": self.uid,
             "batchInput": {
                 "name": name,
                 "dataRowIds": dr_ids,
-                "priority": priority
+                "globalKeys": global_keys,
+                "priority": priority,
+                "consensusSettings": consensus_settings
             }
         }
-
         res = self.client.execute(query_str,
                                   params,
                                   timeout=180.0,
                                   experimental=True)["project"][method]
+        batch = res['batch']
+        batch['size'] = len(dr_ids)
+        return Entity.Batch(self.client,
+                            self.uid,
+                            batch,
+                            failed_data_row_ids=res['failedDataRowIds'])
 
-        res['size'] = len(dr_ids)
-        return Entity.Batch(self.client, self.uid, res)
+    def _create_batch_async(self,
+                            name: str,
+                            dr_ids: Optional[List[str]] = None,
+                            global_keys: Optional[List[str]] = None,
+                            priority: int = 5,
+                            consensus_settings: Optional[Dict[str,
+                                                              float]] = None):
+        method = 'createEmptyBatch'
+        create_empty_batch_mutation_str = """mutation %sPyApi($projectId: ID!, $input: CreateEmptyBatchInput!) {
+                                      project(where: {id: $projectId}) {
+                                        %s(input: $input) {
+                                            id
+                                        }
+                                      }
+                                    }
+                                """ % (method, method)
 
-    def _update_queue_mode(self,
-                           mode: "Project.QueueMode") -> "Project.QueueMode":
+        params = {
+            "projectId": self.uid,
+            "input": {
+                "name": name,
+                "consensusSettings": consensus_settings
+            }
+        }
 
-        if self.queue_mode() == mode:
+        res = self.client.execute(create_empty_batch_mutation_str,
+                                  params,
+                                  timeout=180.0,
+                                  experimental=True)["project"][method]
+        batch_id = res['id']
+
+        method = 'addDataRowsToBatchAsync'
+        add_data_rows_mutation_str = """mutation %sPyApi($projectId: ID!, $input: AddDataRowsToBatchInput!) {
+                                      project(where: {id: $projectId}) {
+                                        %s(input: $input) {
+                                            taskId
+                                        }
+                                      }
+                                    }
+                                """ % (method, method)
+
+        params = {
+            "projectId": self.uid,
+            "input": {
+                "batchId": batch_id,
+                "dataRowIds": dr_ids,
+                "globalKeys": global_keys,
+                "priority": priority,
+            }
+        }
+
+        res = self.client.execute(add_data_rows_mutation_str,
+                                  params,
+                                  timeout=180.0,
+                                  experimental=True)["project"][method]
+
+        task_id = res['taskId']
+
+        task = self._wait_for_task(task_id)
+        if task.status != "COMPLETE":
+            raise LabelboxError(f"Batch was not created successfully: " +
+                                json.dumps(task.errors))
+
+        # obtain batch entity to return
+        get_batch_str = """query %s($projectId: ID!, $batchId: ID!) {
+                          project(where: {id: $projectId}) {
+                             batches(where: {id: $batchId}) {
+                                nodes {
+                                   %s
+                                }
+                             }
+                        }
+                    }
+                    """ % ("getProjectBatchPyApi",
+                           query.results_query_part(Entity.Batch))
+
+        batch = self.client.execute(
+            get_batch_str, {
+                "projectId": self.uid,
+                "batchId": batch_id
+            },
+            timeout=180.0,
+            experimental=True)["project"]["batches"]["nodes"][0]
+
+        # TODO async endpoints currently do not provide failed_data_row_ids in response
+        return Entity.Batch(self.client, self.uid, batch)
+
+    def _update_queue_mode(self, mode: "QueueMode") -> "QueueMode":
+        """
+        Updates the queueing mode of this project.
+
+        Deprecation notice: This method is deprecated. Going forward, projects must
+        go through a migration to have the queue mode changed. Users should specify the
+        queue mode for a project during creation if a non-default mode is desired.
+
+        For more information, visit https://docs.labelbox.com/reference/migrating-to-workflows#upcoming-changes
+
+        Args:
+            mode: the specified queue mode
+
+        Returns: the updated queueing mode of this project
+
+        """
+
+        logger.warning(
+            "Updating the queue_mode for a project will soon no longer be supported."
+        )
+
+        if self.queue_mode == mode:
             return mode
 
-        if mode == Project.QueueMode.Batch:
+        if mode == QueueMode.Batch:
             status = "ENABLED"
-        elif mode == Project.QueueMode.Dataset:
+        elif mode == QueueMode.Dataset:
             status = "DISABLED"
         else:
             raise ValueError(
@@ -648,8 +1010,23 @@ class Project(DbObject, Updateable, Deletable):
 
         return mode
 
-    def queue_mode(self) -> "Project.QueueMode":
-        """Provides the status of if queue mode is enabled in the project."""
+    def get_queue_mode(self) -> "QueueMode":
+        """
+        Provides the queue mode used for this project.
+
+        Deprecation notice: This method is deprecated and will be removed in
+        a future version. To obtain the queue mode of a project, simply refer
+        to the queue_mode attribute of a Project.
+
+        For more information, visit https://docs.labelbox.com/reference/migrating-to-workflows#upcoming-changes
+
+        Returns: the QueueMode for this project
+
+        """
+
+        logger.warning(
+            "Obtaining the queue_mode for a project through this method will soon"
+            " no longer be supported.")
 
         query_str = """query %s($projectId: ID!) {
               project(where: {id: $projectId}) {
@@ -662,9 +1039,9 @@ class Project(DbObject, Updateable, Deletable):
             query_str, {'projectId': self.uid})["project"]["tagSetStatus"]
 
         if status == "ENABLED":
-            return Project.QueueMode.Batch
+            return QueueMode.Batch
         elif status == "DISABLED":
-            return Project.QueueMode.Dataset
+            return QueueMode.Dataset
         else:
             raise ValueError("Status not known")
 
@@ -727,6 +1104,10 @@ class Project(DbObject, Updateable, Deletable):
         Returns:
             bool, indicates if the operation was a success.
         """
+        logger.warning(
+            "LabelingParameterOverrides are deprecated for new projects, and will eventually be removed "
+            "completely. Prefer to use batch based queuing with priority & consensus number of labels instead."
+        )
         self.validate_labeling_parameter_overrides(data)
         data_str = ",\n".join(
             "{dataRow: {id: \"%s\"}, priority: %d, numLabels: %d }" %
@@ -865,6 +1246,81 @@ class Project(DbObject, Updateable, Deletable):
             cursor_path=['project', 'batches', 'pageInfo', 'endCursor'],
             experimental=True)
 
+    def task_queues(self) -> List[TaskQueue]:
+        """ Fetch all task queues that belong to this project
+
+        Returns:
+            A `List of `TaskQueue`s
+        """
+        query_str = """query GetProjectTaskQueuesPyApi($projectId: ID!) {
+              project(where: {id: $projectId}) {
+                taskQueues {
+                  %s
+                }
+              }
+            }
+        """ % (query.results_query_part(Entity.TaskQueue))
+
+        task_queue_values = self.client.execute(
+            query_str, {"projectId": self.uid},
+            timeout=180.0,
+            experimental=True)["project"]["taskQueues"]
+
+        return [
+            Entity.TaskQueue(self.client, field_values)
+            for field_values in task_queue_values
+        ]
+
+    def move_data_rows_to_task_queue(self, data_row_ids: List[str],
+                                     task_queue_id: str):
+        """
+
+        Moves data rows to the specified task queue.
+
+        Args:
+            data_row_ids: a list of data row ids to be moved
+            task_queue_id: the task queue id to be moved to, or None to specify the "Done" queue
+
+        Returns:
+            None if successful, or a raised error on failure
+
+        """
+        method = "createBulkAddRowsToQueueTask"
+        query_str = """mutation AddDataRowsToTaskQueueAsyncPyApi(
+          $projectId: ID!
+          $queueId: ID
+          $dataRowIds: [ID!]!
+        ) {
+          project(where: { id: $projectId }) {
+            %s(
+              data: { queueId: $queueId, dataRowIds: $dataRowIds }
+            ) {
+              taskId
+            }
+          }
+        }
+        """ % method
+
+        task_id = self.client.execute(
+            query_str, {
+                "projectId": self.uid,
+                "queueId": task_queue_id,
+                "dataRowIds": data_row_ids
+            },
+            timeout=180.0,
+            experimental=True)["project"][method]["taskId"]
+
+        task = self._wait_for_task(task_id)
+        if task.status != "COMPLETE":
+            raise LabelboxError(f"Data rows were not moved successfully: " +
+                                json.dumps(task.errors))
+
+    def _wait_for_task(self, task_id: str) -> Task:
+        task = Task.get_task(self.client, task_id)
+        task.wait_till_done()
+
+        return task
+
     def upload_annotations(
             self,
             name: str,
@@ -932,6 +1388,54 @@ class Project(DbObject, Updateable, Deletable):
             raise ValueError(
                 f'Invalid annotations given of type: {type(annotations)}')
 
+    def _wait_until_data_rows_are_processed(
+            self,
+            data_row_ids: Optional[List[str]] = None,
+            global_keys: Optional[List[str]] = None,
+            wait_processing_max_seconds: int = _wait_processing_max_seconds,
+            sleep_interval=30):
+        """ Wait until all the specified data rows are processed"""
+        start_time = datetime.now()
+
+        while True:
+            if (datetime.now() -
+                    start_time).total_seconds() >= wait_processing_max_seconds:
+                raise ProcessingWaitTimeout(
+                    "Maximum wait time exceeded while waiting for data rows to be processed. Try creating a batch a bit later"
+                )
+
+            all_good = self.__check_data_rows_have_been_processed(
+                data_row_ids, global_keys)
+            if all_good:
+                return
+
+            logger.debug(
+                'Some of the data rows are still being processed, waiting...')
+            time.sleep(sleep_interval)
+
+    def __check_data_rows_have_been_processed(
+            self,
+            data_row_ids: Optional[List[str]] = None,
+            global_keys: Optional[List[str]] = None):
+
+        if data_row_ids is not None and len(data_row_ids) > 0:
+            param_name = "dataRowIds"
+            params = {param_name: data_row_ids}
+        else:
+            param_name = "globalKeys"
+            global_keys = global_keys if global_keys is not None else []
+            params = {param_name: global_keys}
+
+        query_str = """query CheckAllDataRowsHaveBeenProcessedPyApi($%s: [ID!]) {
+            queryAllDataRowsHaveBeenProcessed(%s:$%s) {
+                allDataRowsHaveBeenProcessed
+           }
+        }""" % (param_name, param_name, param_name)
+
+        response = self.client.execute(query_str, params)
+        return response["queryAllDataRowsHaveBeenProcessed"][
+            "allDataRowsHaveBeenProcessed"]
+
 
 class ProjectMember(DbObject):
     user = Relationship.ToOne("User", cache=True)
